package org.netkernel.rdf.jena.endpoint;

import java.io.ByteArrayOutputStream;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.rdf.jena.rep.IRepJenaModel;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.shared.Lock;

public class RDFSerializerAccessor extends StandardAccessorImpl
{
	public RDFSerializerAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	IRepJenaModel jmr=context.source("arg:operand", IRepJenaModel.class);
		ByteArrayOutputStream baos=new ByteArrayOutputStream(2048);
		String type=context.getThisRequest().getArgumentValue("activeType");
		Model m=jmr.getModelReadOnly();
		Lock lock=m.getLock();
		String mime=null;
		lock.enterCriticalSection(Lock.READ);
		try
		{	if(type.equals("jRDFSerializeXML"))
			{	m.write(baos);
				mime="application/rdf+xml";
			}
			else if (type.equals("jRDFSerializeN3"))
			{	m.write(baos, "N3");
				mime="text/rdf+n3";
			}
			else if (type.equals("jRDFSerializeN-TRIPLE"))
			{	m.write(baos, "N-TRIPLE");
				mime="text/plain";
			}
			else if (type.equals("jRDFSerializeTURTLE"))
			{	m.write(baos, "TURTLE");
				mime="text/turtle";
			}
		}
		finally
		{	lock.leaveCriticalSection();
		}
		ByteArrayRepresentation bar=new ByteArrayRepresentation(baos.toByteArray());
		INKFResponse resp=context.createResponseFrom(bar);
		resp.setMimeType(mime);
	}
}
